import numpy as np

from javelin.lcmodel import Cont_Model, Pmap_Model, Rmap_Model
from javelin.zylc import get_data

import pypetal_jav.defaults as dft
from pypetal_jav.petalio import write_data




def fix_jav_params_after_ufj(javelin_params, drw_rej_res):

    nlc = len(drw_rej_res['reject_data'])
    javelin_params = dft.set_javelin(javelin_params, nlc, ret_dict=True)
    new_params = javelin_params.copy()

    drw_sigma = np.median(drw_rej_res['sigmas'][0])
    drw_tau = np.median(drw_rej_res['taus'][0])

    if javelin_params['rm_type'] == 'spec':
        ntophat = 3
    elif javelin_params['rm_type'] == 'phot':
        ntophat = 4


    if javelin_params['together']:

        if javelin_params['fixed'] is None:
            fixed_tot = np.ones(2 + (nlc-1)*ntophat, dtype=int)
            p_fix_tot = np.ones(2 + (nlc-1)*ntophat )
        else:
            fixed_tot = javelin_params['fixed']
            p_fix_tot = javelin_params['p_fix']


        fixed_tot[0] = 0
        p_fix_tot[0] = np.log(drw_sigma)

        fixed_tot[1] = 0
        p_fix_tot[1] = np.log(drw_tau)


    else:        
        fixed_tot = np.ones( ( nlc-1, 2 + ntophat ), dtype=int )
        p_fix_tot = np.ones( ( nlc-1, 2 + ntophat ) )

        if javelin_params['fixed'] is not None:
            for i in range(nlc-1):
                if javelin_params['fixed'][i] is None:
                    continue
                else:
                    fixed_tot[i,:] = javelin_params['fixed'][i]
                    p_fix_tot[i,:] = javelin_params['p_fix'][i]


        for i in range(nlc-1):
            fixed_tot[i,0] = 0
            p_fix_tot[i,0] = np.log(drw_sigma)

            fixed_tot[i,1] = 0
            p_fix_tot[i,1] = np.log(drw_tau)


    new_params['fixed'] = fixed_tot
    new_params['p_fix'] = p_fix_tot

    return new_params



def get_javelin_filenames(output_chain, output_burn, output_logp, prefix, output_dir=None):
    tot_fnames = np.full(3, None)

    if output_chain:
        tot_fnames[0] = 'chain_' + prefix + '.txt'

    if output_burn:
        tot_fnames[1] = 'burn_' + prefix + '.txt'

    if output_logp:
        tot_fnames[2] = 'logp_' + prefix + '.txt'

    if output_dir is not None:
        if output_dir[-1] != r'/':
            output_dir += r'/'

        for i in range(len(tot_fnames)):

            if tot_fnames[i] is not None:
                tot_fnames[i] = output_dir + tot_fnames[i]

    return tot_fnames



def run_javelin(cont_fname, line_fnames, line_names,
                rm_type='spec',
                lagtobaseline=0.3, laglimit='baseline',
                fixed=None, p_fix=None, subtract_mean=True,
                nwalkers=100, nburn=100, nchain=100, threads=1, output_chains=False,
                output_burn=False, output_logp=False, output_dir=None,
                nbin=50, verbose=False, plot=False):

    """Run JAVELIN on a set of light curves.

    Parameters
    ----------
    cont_fname : str
        The filename of the continuum light curve.
    line_fnames: str, list of str
        The filename(s) of the line light curve(s).
    rm_type : str, optional
        The type of analysis (and JAVELIN model) to use. May either be "spec" for
        spectroscopic RM or "phot" for photometric RM. Default is "spec".
    lagtobaseline : float, optional
        JAVELIN will use a log prior on the lag and penalize lags larger x*baseline,
        where x is the input value and baseline is the baseline of the light curves.
        Default is 0.3.
    laglimit : (2,) list of floats, str, optional
         The range of lags to search for the lag between light curves in the following form:
         [lower, upper]. If set to "baseline", the range will be set to [-baseline, baseline].
         Default is 'baseline'.
    fixed : None or list of floats, optional
        An array defining which parameters are fixed and which are allowed to vary. The length of
        the array must match the number of parameters ( 2 + 3*len(line_fnames) ). In the array,
        1 corresponds to a variable parameter and 0 corresponds to a fixed parameter. If set to ``None``,
        all parameters will be allowed to vary. Default is ``None``.
    p_fix : None or list of floats, optional
        An array defining the values of the fixed parameters. The length of the array must match the
        number of parameters and of the ``fixed" array. If set to ``None``, all parameters will be allowed
        to vary. Must be defined if ``fixed`` is. Default is ``None``.
    subtract_mean : bool, optional
        If True, will subtract the mean of the light curves before analysis. Default is ``True``.
    nwalkers : int, optional
        The number of walkers to use in the MCMC. Default is 100.
    nburn : int, optional
        The number of burn-in steps to use in the MCMC. Default is 100.
    nchain : int, optional
        The number of steps to use in the MCMC. Default is 100.
    threads : int, optional
        The number of parallel threads to use in the MCMC. Default is 1.
    output_chains : bool, optional
        If ``True``, will output the MCMC chains to a file. Default is ``False``.
    output_burn : bool, optional
        If ``True``, will output the MCMC burn-in chains to a file. Default is ``False``.
    output_logp : bool, optional
        If ``True``, will output the MCMC log-probability values to a file. Default is ``False``.

    Returns
    -------
    res : dict
        A dictionary containing the results of the JAVELIN analysis. Has the following keys:
        * cont_hpd : list of float
            The highest posterior density (HPD) interval for the continuum light curve DRW fits.
        * tau : list of float
            The MC chain for the DRW tau parameter.
        * sigma : list of float
            The MC chain for the DRW sigma parameter.
        * tophat_params : list of float
            The MC chains for the tophat parameters (lag, width, scale), 3 for each light curve.
        * hpd : list of float
            The HPD intervals for the DRW and tophat parameters for each light curve.
        * cont_model : javelin.lcmodel.Cont_Model
            The continuum model object.
        * rmap_model : javelin.lcmodel.Rmap_Model, javelin.lcmodel.Pmap_Model
            The RM model object.
        * cont_dat : javelin.zylc.LightCurve
            The continuum light curve object.
        * tot_dat : javelin.zylc.LightCurve
            The object containing all light curves (continuum + lines).
    """


    total_fnames = np.hstack( [cont_fname, line_fnames] )
    total_names = line_names

    #Deal with csv files
    xi, yi, erri = np.loadtxt(cont_fname, delimiter=',', unpack=True, usecols=[0,1,2])
    write_data( [xi, yi, erri] , output_dir + 'cont_lcfile.dat', delimiter='\t' )

    for i in range(len(total_fnames[1:])):
        xi, yi, erri = np.loadtxt(total_fnames[i+1], delimiter=',', unpack=True, usecols=[0,1,2])
        write_data( [xi, yi, erri], output_dir + line_names[i+1] + '_lcfile.dat', delimiter='\t' )

    cont_fname = output_dir + 'cont_lcfile.dat'
    total_fnames = np.concatenate( [ [cont_fname], [output_dir + x + '_lcfile.dat' for x in line_names[1:] ]])

    con_dat = get_data(cont_fname, names=[total_names[0]], set_subtractmean=subtract_mean)
    tot_dat = get_data(total_fnames, names=total_names, set_subtractmean=subtract_mean)

    if fixed is not None:
        if (fixed[0] == 0) & (fixed[1] == 0):
            skip_init = True
        else:
            skip_init = False
    else:
        skip_init = False


    if not skip_init:

        #Run continuum fit to get priors on DRW values
        fnames = get_javelin_filenames(output_chains, output_burn, output_logp, 'cont', output_dir)

        if fixed is not None:
            fixed_cont = fixed[:2]
            p_fix_cont = p_fix[:2]
        else:
            fixed_cont = None
            p_fix_cont = None

        cmod = Cont_Model(con_dat)
        cmod.do_mcmc(fixed=fixed_cont, p_fix=p_fix_cont,
                    nwalkers=nwalkers, nburn=nburn, nchain=nchain, threads=1,
                    fchain=fnames[0], fburn=fnames[1], flogp=fnames[2],
                    set_verbose=verbose)

        #Get HPD from continuum fit
        cmod.get_hpd(set_verbose=False)
        conthpd = cmod.hpd

        if fixed is not None:
            if fixed[0] == 0:
                conthpd[0,0] = conthpd[1,0] - 1e-10
                conthpd[2,0] = conthpd[1,0] + 1e-10

            if fixed[1] == 0:
                conthpd[0,1] = conthpd[1,1] - 1e-10
                conthpd[2,1] = conthpd[1,1] + 1e-10

        #Get histogram figure from continuum fit
        if plot:
            cmod.show_hist(bins=nbin)
    else:
        conthpd = None
        cmod = None






    #Run fit on continuum + line(s)
    fnames = get_javelin_filenames(output_chains, output_burn, output_logp, 'rmap', output_dir)

    if rm_type == 'spec':
        rmod = Rmap_Model(tot_dat)
    elif rm_type == 'phot':
        rmod = Pmap_Model(tot_dat)

    if (len(total_fnames) == 2) & (len(laglimit) == 2):
        laglimit = [laglimit]

    rmod.do_mcmc(conthpd=conthpd, fixed=fixed, p_fix=p_fix, lagtobaseline=lagtobaseline, laglimit=laglimit,
                 nwalkers=nwalkers, nburn=nburn, nchain=nchain,
                 threads=threads, fchain=fnames[0], fburn=fnames[1], flogp=fnames[2],
                 set_verbose=verbose)

    #Get HPD from continuum + line(s) fit
    rmod.get_hpd(set_verbose=False)
    rmap_hpd = rmod.hpd

    tau = np.exp( rmod.flatchain[:,1] )
    sigma = np.exp( rmod.flatchain[:,0] )
    tophat_params = rmod.flatchain[:, 2:].T


    return {
        'cont_hpd': conthpd,
        'tau': tau,
        'sigma': sigma,
        'tophat_params': tophat_params,
        'hpd': rmap_hpd,
        'cont_model': cmod,
        'rmap_model': rmod,
        'cont_dat': con_dat,
        'tot_dat': tot_dat
    }



def javelin_pred_lc(rmod, t_cont, t_lines, nbin=None, metric='med'):

    """Predict light curve(s) from a JAVELIN RM fit object.
    Parameters
    ----------
    rmod : javelin.lcmodel.Rmap_Model or javelin.lcmodel.Pmap_Model
        The output RM model from ``run_javelin``, after fitting the data.
    t_cont : array_like
        The time array for the continuum light curve.
    t_lines : array_like
        The time array for the line light curve(s).
    metric : str, optional
        The metric to use to get the bestfit parameter from the parameter distributions.
        Can be either "mean" or "med". Default is "med".
    Returns
    -------
    rmap_bestfit : javelin.lcmodel.Rmap_Model, javelin.lcmodel.Pmap_Model
        The RM model using the best fit parameters to predict the light curve(s).
    """

    if type(rmod) == Rmap_Model:
        rm_type = 'spec'
    elif type(rmod) == Pmap_Model:
        rm_type = 'phot'

    tau = rmod.flatchain[:,1]
    sigma = rmod.flatchain[:,0]
    tophat_params = rmod.flatchain[:, 2:].T

    if rm_type == 'spec':
        Nlc = len(tophat_params)//3
        ntophat = 3
    elif rm_type == 'phot':
        Nlc = len(tophat_params)//4
        ntophat = 4

    if metric == 'med':
        func = np.median
    if metric == 'mean':
        func = np.mean



    tau_best = func(tau)
    sigma_best = func(sigma)
    tophat_best = np.zeros( Nlc*ntophat )


    for i in range(Nlc):
        for j in range(3):


            if j == 0:
                mc_vals = tophat_params[ntophat*i + j, :]
                tophat_best[ ntophat*i + j ] = func(mc_vals)

            else:
                tophat_best[ ntophat*i + j ] = func(tophat_params[ ntophat*i + j, : ])


    bestfit_vals = np.concatenate([ [sigma_best], [tau_best], tophat_best ])
    rmap_bestfit = rmod.do_pred(bestfit_vals)

    return rmap_bestfit



import numpy as np
from astropy.table import Table



def merge_dicts(x, y):
    z = x.copy()
    z.update(y)
    return z



def set_general(input_args, fnames):

    default_kwargs = {
        'verbose': False,
        'plot': False,
        'time_unit': 'd',
        'lc_unit': 'Arbitrary Units',
        'file_fmt': 'csv',
        'lag_bounds': 'baseline',
        'threads': 1
    }

    params = merge_dicts(default_kwargs, input_args)

    verbose = params['verbose']
    plot = params['plot']
    time_unit = params['time_unit']
    lc_unit = params['lc_unit']
    file_fmt = params['file_fmt']
    lag_bounds = params['lag_bounds']
    threads = params['threads']

    if isinstance(lag_bounds, list):
        if len(lag_bounds) == 1:
            assert isinstance(lag_bounds[0], list) or isinstance(lag_bounds[0], str) or ( lag_bounds[0] is None )

        elif len(lag_bounds) == 2:
            pass

        else:
            for i in range(len(lag_bounds)):
                assert len(lag_bounds[i]) == 2


    if isinstance(lc_unit, str):
        lc_unit = list( np.full( len(fnames), lc_unit ) )




    len_lag_bounds = len(lag_bounds)
    if len_lag_bounds == 2:
        if not ( isinstance(lag_bounds[0], list) or ( lag_bounds[0] == 'baseline' ) ):
            len_lag_bounds = 1
        else:
            len_lag_bounds = 2
            
    if lag_bounds is None:
        lag_bounds = [None] * (len(fnames)-1)

    if len_lag_bounds != len(fnames)-1:

        lag_bounds_og = lag_bounds
        lag_bounds = []

        for i in range(len(fnames)-1):
            lag_bounds.append(lag_bounds_og)

    if (len(fnames) == 2) and ( not isinstance(lag_bounds[0], list) ):
        if not isinstance(lag_bounds[0], str):
            lag_bounds = [lag_bounds]



    xvals_tot = []
    baselines = []
    for i in range(len(fnames)):

        try:
            dat = Table.read(fnames[i], format=file_fmt)
        except:
            dat = Table.read(fnames[i], format='ascii')

        colnames = dat.colnames
        x = np.array( dat[colnames[0]] )

        sort_ind = np.argsort(x)
        x = x[sort_ind]

        xvals_tot.append(x)

    for i in range(len(fnames)-1):
        baseline = np.max([ xvals_tot[0].max(), xvals_tot[i+1].max() ]) - np.min([ xvals_tot[0].min(), xvals_tot[i+1].min() ])
        baselines.append(baseline)

    for i in range(len(fnames)-1):
        if (lag_bounds[i] is None) | (lag_bounds[i] == 'baseline'):
            lag_bounds[i] = [-baselines[i], baselines[i]]


    #Return dict so it can be passed to other functions
    output = {
        'verbose': verbose,
        'plot': plot,
        'time_unit': time_unit,
        'lc_unit': lc_unit,
        'file_fmt': file_fmt,
        'lag_bounds': lag_bounds,
        'threads': threads
    }

    return output



def set_javelin(input_args, nlc, ret_dict=False):

    default_kwargs = {
        'lagtobaseline': 0.3,
        'fixed': None,
        'p_fix': None,
        'subtract_mean': True,
        'nwalker': 100,
        'nburn': 100,
        'nchain': 100,
        'output_chains': True,
        'output_burn': True,
        'output_logp': True,
        'nbin': 50,
        'metric': 'med',
        'together': False,
        'rm_type': 'spec'
    }

    params = merge_dicts(default_kwargs, input_args)

    lagtobaseline = params['lagtobaseline']
    fixed = params['fixed']
    p_fix = params['p_fix']
    subtract_mean = params['subtract_mean']
    nwalkers = params['nwalker']
    nburn = params['nburn']
    nchain = params['nchain']
    output_chains = params['output_chains']
    output_burn = params['output_burn']
    output_logp = params['output_logp']
    nbin = params['nbin']
    metric = params['metric']
    together = params['together']
    rm_type = params['rm_type']

    if (rm_type == 'phot') & (together):
        print('ERROR: JAVELIN cannot do phtotometric RM with more than two lines.')
        print('Setting together=False')
        together = False

    if not together:

        if fixed is not None:
            if len(fixed) != nlc-1:

                fixed_og = fixed
                p_fix_og = p_fix

                fixed = []
                p_fix = []
                for _ in range(nlc-1):
                    fixed.append(fixed_og)
                    p_fix.append(p_fix_og)

        else:
            fixed = np.full( nlc-1, None )
            p_fix = np.full( nlc-1, None )

        assert len(fixed) == nlc-1

    else:
        if fixed is not None:
            assert len(fixed) == 2 + 3*( nlc - 1 )

    if ret_dict:
        return {
            'lagtobaseline': lagtobaseline,
            'fixed': fixed,
            'p_fix': p_fix,
            'subtract_mean': subtract_mean,
            'nwalker': nwalkers,
            'nburn': nburn,
            'nchain': nchain,
            'output_chains': output_chains,
            'output_burn': output_burn,
            'output_logp': output_logp,
            'nbin': nbin,
            'metric': metric,
            'together': together,
            'rm_type': rm_type
        }

    else:
        return lagtobaseline, fixed, p_fix, subtract_mean, \
            nwalkers, nburn, nchain, output_chains, \
                output_burn, output_logp, nbin, metric, together, rm_type



import os
import glob

import numpy as np

import pypetal_jav.modules as modules

from pypetal_jav import defaults
from pypetal_jav.petalio import make_directories
from pypetal_jav.utils import fix_jav_params_after_ufj



def run_pipeline(output_dir, line_names=None,
                 javelin_params={}, use_for_javelin=False, 
                 drw_rej_res={}, **kwargs):


    output_dir = os.path.abspath(output_dir) + r'/'

    #pyPetal will create the output directory if it doesn't exist
    #Also will create the javelin subdirectories
    if not os.path.exists(output_dir):
        raise Exception('This assumes that pypetal.pipeline.run_pipeline has already been run.')

    if line_names is None:
        light_curve_fnames = glob.glob(output_dir + 'light_curves/*.dat')
        line_names = [os.path.basename(fname).split('.')[0] for fname in light_curve_fnames]


    #Look for light curve filenames
    line_fnames = []
    if output_dir + 'processed_lcs/' in glob.glob(output_dir +'*/'):
        fnames = [output_dir + 'processed_lcs/' + name + '_data.dat' for name in line_names]
    else:
        fnames = [output_dir + 'light_curves/' + name + '.dat' for name in line_names]

    cont_fname = fnames[0]
    line_fnames = fnames[1:]


    if type(line_fnames) is str:
        line_fnames = [line_fnames]


    #Read in general kwargs
    general_kwargs = defaults.set_general(kwargs, fnames)

    #Get "together"
    _, _, _, _, _, _, _, _, _, _, _, _, together, _ = defaults.set_javelin(javelin_params, len(fnames) )

    make_directories(output_dir, line_names, together)

    #Name lines if unnamed
    if line_names is None:
        line_names = np.zeros( len(line_fnames), dtype=str )
        for i in range(len(line_fnames)):
            line_names.append('Line {}'.format(i+1))


    if use_for_javelin:
        javelin_params = fix_jav_params_after_ufj(javelin_params, drw_rej_res)

    javelin_res = modules.javelin_tot(cont_fname, line_fnames, line_names, output_dir, general_kwargs, javelin_params)

    return javelin_res



import corner
import javelin.lcmodel
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
import palettable
from matplotlib.colors import ListedColormap


mpl.rcParams['xtick.minor.visible'] = True
mpl.rcParams['xtick.top'] = True
mpl.rcParams['xtick.direction'] = 'in'

mpl.rcParams['ytick.minor.visible'] = True
mpl.rcParams['ytick.right'] = True
mpl.rcParams['ytick.direction'] = 'in'

mpl.rcParams["figure.autolayout"] = False

mpl.rcParams['savefig.dpi'] = 300
mpl.rcParams['savefig.format'] = 'pdf'


def plot_javelin_hist(res, fixed=None, nbin=50,
                      time_unit='d',
                      remove_fixed=True, fname=None):


    """Plot the histograms of the posteriors for the JAVELIN fit parameters.
    Parameters
    ----------
    res : dict
        The output of ``pypetal.modules.run_javelin``.
    fixed : dict, optional
        The ``fixed`` argument passed to JAVELIN. If ``None``, all parameters will be assumed to vary. Default is ``None``.
    nbin : int, optional
        The number of bins to use for the histograms. Default is 50.
    time_unit : str, optional
        The unit of time for the light curves. Default is 'd'.
    remove_fixed : bool, optional
        If ``True``, will remove the fixed parameters from the plot. Default is ``True``.
    fname : str, optional
        If not ``None``, will save the plot to the given filename. Default is ``None``.
    Returns
    -------
    fig : matplotlib.figure.Figure
        The figure object.
    ax : list of matplotlib.axes.Axes
        The axes for the plot.
    """


    if type(res['rmap_model']) == javelin.lcmodel.Rmap_Model:
        rm_type = 'spec'
        Ncol = 3
    elif type(res['rmap_model']) == javelin.lcmodel.Pmap_Model:
        rm_type = 'phot'
        Ncol = 4


    time_unit_txt = '[' + str(time_unit) + ']'

    Nrow = len(res['tophat_params'])//3 + 1

    names = res['tot_dat'].names[1:]
    x1 = res['tot_dat'].jlist[0]

    if fixed is not None:
        fixed_reshape = np.zeros( (Nrow, Ncol) )
        fixed_reshape[0,0] = fixed[0]
        fixed_reshape[0,1] = fixed[1]

        for i in range(Nrow-1):
            for j in range(Ncol):
                fixed_reshape[i+1,j] = fixed[2 + j + i*Ncol ]

    else:
        fixed_reshape = np.ones( (Nrow, Ncol) )
        fixed_reshape[0, 2] = 0


    fig, ax = plt.subplots(Nrow, Ncol, figsize=(5*Ncol, 5*Nrow))

    ax[0,0].hist( np.log10(res['sigma']), bins=nbin )
    ax[0,1].hist( np.log10(res['tau']), bins=nbin )


    #Plot tophat params
    for i in range(1, Nrow):
        for j in range(Ncol):
            vals = res['tophat_params'][ (i-1)*Ncol + j ]
            x2 = res['tot_dat'].jlist[i]

            ax[i,j].hist( vals, bins=nbin )


    ax[0,0].set_xlabel(r'$\log_{10}(\sigma_{\rm DRW})$', fontsize=19)
    ax[0,1].set_xlabel(r'$\log_{10}(\tau_{\rm DRW} \,\, ' + time_unit_txt + ')$', fontsize=19)

    for i in range(1, Nrow):
        for j in range(Ncol):

            if j == 0:
                ax[i,j].set_xlabel(r'$t_{' + names[i-1] + '}$ ' + time_unit_txt, fontsize=22)
            if j == 1:
                ax[i,j].set_xlabel(r'w$_{' + names[i-1] + '}$ ' + time_unit_txt, fontsize=22)
            if j == 2:
                ax[i,j].set_xlabel(r's$_{' + names[i-1] + '}$', fontsize=22)
            if j == 3:
                ax[i,j].set_xlabel(r'$\alpha_{' + names[i-1] + '}$', fontsize=22)

    for i in range(Nrow):
        ax[i, 0].set_ylabel('N', fontsize=19)

    for i in range(2, Ncol):
        ax[0, i].axis('off')


    for i in range(Nrow):
        for j in range(Ncol):
            ax[i,j].tick_params('both', labelsize=14)
            ax[i,j].tick_params('both', which='major', length=8)
            ax[i,j].tick_params('both', which='minor', length=4)

            if remove_fixed:
                if fixed_reshape[i,j] == 0:
                    ax[i,j].clear()
                    ax[i,j].axis('off')

    plt.subplots_adjust( wspace=.25, hspace=.25 )

    if fname is not None:
        plt.savefig( fname, dpi=200, bbox_inches='tight' )

    return fig, ax


def javelin_corner(res, nbin=20, fname=None):

    """Create a corner plot for the JAVELIN parameter results.
    Parameters
    ----------
    res : dict
        The output of ``pypetal.modules.run_javelin``.
    nbins : int, optional
        The number of bins to use for the histograms. Default is 20.
    fname : str, optional
        If not ``None``, will save the plot to the given filename. Default is ``None``.
    Returns
    -------
    fig : matplotlib.figure.Figure
        The figure object for the plot.
    ax : list of matplotlib.axes.Axes
        The axes objects for the plot.
    """

    if type(res['rmap_model']) == javelin.lcmodel.Rmap_Model:
        rm_type = 'spec'
    elif type(res['rmap_model']) == javelin.lcmodel.Pmap_Model:
        rm_type = 'phot'


    labels = []
    labels.append( r'$\log_{10} (\sigma_{\rm DRW})$' )
    labels.append( r'$\log_{10} (\tau_{\rm DRW})$' )

    for i in range( res['tot_dat'].nlc - 1 ):
        labels.append( r'$t_{' + res['tot_dat'].names[i+1] + '}$' )
        labels.append( r'$w_{' + res['tot_dat'].names[i+1] + '}$' )
        labels.append( r'$s_{' + res['tot_dat'].names[i+1] + '}$' )

        if rm_type == 'phot':
            labels.append( r'$\alpha_{' + res['tot_dat'].names[i+1] + '}$' )


    #Plot original output with weighted output superposed on histograms
    corner_dat = np.vstack( [np.log10(res['sigma']), np.log10(res['tau']),  res['tophat_params']]).T

    fig = corner.corner( corner_dat,
                labels=labels, show_titles=False, bins=nbin,
                label_kwargs={'fontsize': 20} )

    ax = np.array(fig.axes).reshape( (2 + len(res['tophat_params']), 2 + len(res['tophat_params']) ) )

    if fname is not None:
        plt.savefig( fname, dpi=200, bbox_inches='tight' )

    return fig, ax



def plot_javelin_bestfit(res, bestfit_model, time_unit='d', lc_unit='mag', fname=None):


    """Plot the fit to the data using the best-fit JAVELIN parameters.
    Parameters
    ----------
    res : dict
        The output of ``pypetal.modules.run_javelin``.
    bestfit_model : dict
        The bestfit model after using ``pypetal.utils.javelin_pred_lc``.
    time_unit : str, optional
        The time unit for the light curves. Default is 'd'.
    lc_unit : str, optional
        The light curve unit. Default is 'mag'.
    fname : str, optional
        If not ``None``, will save the plot to the given filename. Default is ``None``.
    Returns
    -------
    fig : matplotlib.figure.Figure
        The figure object for the plot.
    ax : list of matplotlib.axes.Axes
        The axes objects for the plot.
    """

    if isinstance(lc_unit, str):
        lc_unit = np.full( tot_dat.nlc, lc_unit )


    cmap = ListedColormap( palettable.cartocolors.qualitative.Vivid_10.mpl_colors )
    colors = cmap.colors

    tot_dat = res['tot_dat']

    xmin = np.min( [bestfit_model.jlist[i][0] for i in range( bestfit_model.nlc ) ] )
    xmax = np.min( [bestfit_model.jlist[i][-1] for i in range( bestfit_model.nlc ) ] )



    fig, ax = plt.subplots(tot_dat.nlc, 1, sharex=True, figsize=(13, 7))
    for i in range(len(ax)):
        ax[i].set_prop_cycle('color', palettable.cartocolors.qualitative.Bold_10.mpl_colors )


    for i in range(tot_dat.nlc):
        ax[i].errorbar( tot_dat.jlist[i], tot_dat.mlist[i] + tot_dat.blist[i],
                    yerr=tot_dat.elist[i], fmt='.', ms=10, mfc=colors[9], mec='k',
                    label='Data', zorder=0 )

        ax[i].plot( bestfit_model.jlist[i], bestfit_model.mlist[i]+bestfit_model.blist[i],
                    c=colors[i % 9], lw=2, label='Best fit', zorder=-1 )
        ax[i].fill_between( bestfit_model.jlist[i],
                        bestfit_model.mlist[i]+bestfit_model.blist[i]+bestfit_model.elist[i],
                        bestfit_model.mlist[i]+bestfit_model.blist[i]-bestfit_model.elist[i],
                        color=colors[i % 9], alpha=.3, zorder=-2)


        y1, y2 = ax[i].get_ylim()
        y2 = y1 + (y2 - y1)*1.25
        ax[i].set_ylim( y1, y2 )

        ax[i].set_xlim( xmin, xmax )
        ax[i].text( .02, .8, tot_dat.names[i], transform=ax[i].transAxes, fontsize=16 )

        ax[i].tick_params('both', labelsize=15)
        ax[i].tick_params('both', which='major', length=8)
        ax[i].tick_params('both', which='minor', length=5)

        ax[i].legend( bbox_to_anchor=(1.13, 1.03), fontsize=12 )



        if lc_unit[i] == 'mag':
            ytxt = 'Magnitude'
            ax[i].invert_yaxis()

        elif lc_unit[i] == 'Arbitrary Units':
            ytxt = 'Flux'
        else:
            ytxt = 'Flux [' + str(lc_unit[i]) + ']'

        ax[i].set_ylabel( ytxt, fontsize=22, va='center' )

    ax[-1].set_xlabel('Time [' + str(time_unit) + ']', fontsize=20)

    plt.subplots_adjust(hspace=0)

    if fname is not None:
        plt.savefig( fname, dpi=200, bbox_inches='tight' )

    return fig, ax



from .defaults import *
from .modules import *
from .petalio import *
from .pipeline import *
from .plotting import *
from .utils import *

__all__ = ["pypetal_jav.defaults", "pypetal_jav.modules", "pypetal_jav.petalio",
           "pypetal_jav.pipeline", "pypetal_jav.plotting", "pypetal_jav.utils"]



from setuptools import setup

NAME = 'pypetal-jav'
VERSION = '0.1.0'
URL = 'https://github.com/Zstone19/pypetal-jav'

AUTHOR = 'Zachary Stone'
EMAIL = 'stone28@illinois.edu'

#Get requirements.txt
REQUIREMENTS = []
with open('requirements.txt', 'r') as f:

    for line in f.readlines():
        REQUIREMENTS.append( line.strip('\n\r')  )




setup(
    name=NAME,
    version=VERSION,
    url='https://github.com/Zstone19/pypetal-jav',
    author=AUTHOR,
    author_email=EMAIL,
    maintainer=AUTHOR,
    maintainer_email=EMAIL,
    license='MIT',
    install_requires=REQUIREMENTS,
    python_requires='<3.0',
    packages=['pypetal_jav'],
    package_dir={'pypetal_jav':'./src/pypetal-jav'},
    include_package_data=True
)



import errno
import os
import numpy as np



def makedirs(dirname, exist_ok=False):
    try:
        os.makedirs(dirname)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise


def make_directories(output_dir, line_names, together):

    #Create subdirectories for each line and javelin
    for i in range(len(line_names)):
        makedirs( output_dir + line_names[i], exist_ok=True )

    for i in range(len(line_names)-1):

        if together:
            makedirs( output_dir + 'javelin/', exist_ok=True )
        else:
            makedirs( output_dir + line_names[i+1] + '/javelin', exist_ok=True )

    return



def write_data(arr, fname, header=None, delimiter=','):

    arr = np.array(arr, dtype=object)

    ndim = len( arr.shape )
    assert ndim <= 2


    if ndim == 2:
        cols = len(arr)
        rows = len(arr[0])

        with open(fname, "w") as file:

            if header is not None:
                file.write(header + "\n")

            for i in range(rows):

                string = "{}".format(arr[0][i]) + delimiter
                for j in range(1, cols-1):
                    string += "{}".format(arr[j][i]) + delimiter

                string += "{}\n".format(arr[-1][i])

                file.write(string)

    elif ndim == 1:
        rows = len(arr)

        with open(fname, "w") as file:

            if header is not None:
                file.write(header + "\n")

            for i in range(rows):
                file.write( "{}\n".format(arr[i]) )

    return



import os

import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import numpy as np

import pypetal_jav.defaults as defaults
import pypetal_jav.plotting as plotting
import pypetal_jav.utils as utils
from pypetal_jav.petalio import write_data


def javelin_tot(cont_fname, line_fnames, line_names, output_dir, general_kwargs, kwargs):

    if line_fnames is str:
        line_fnames = [line_fnames]

    #--------------------------------------------------
    #Read general kwargs

    verbose = general_kwargs['verbose']
    plot = general_kwargs['plot']
    time_unit = general_kwargs['time_unit']
    lc_unit = general_kwargs['lc_unit']
    laglimit = general_kwargs['lag_bounds']
    threads = general_kwargs['threads']

    #--------------------------------------------------
    #Read kwargs

    lagtobaseline, fixed, p_fix, subtract_mean, \
        nwalkers, nburn, nchain, output_chains, \
            output_burn, output_logp, nbin, metric, \
                together, rm_type = defaults.set_javelin( kwargs, len(line_names) )

    #--------------------------------------------------
    #Account for parameters if javelin['together'] = False
    if not together:

        if ( type(laglimit) is str ) | ( laglimit is None ):
            laglimit = np.full( len(line_fnames), laglimit )

    #--------------------------------------------------

    if ( not isinstance(laglimit, str) ) & ( laglimit is not None ):
        if len(laglimit) > 2:
            laglimit_str = 'array'
        else:
            laglimit_str = laglimit
    else:
        laglimit_str = laglimit


    if verbose:
        txt_str = """
Running JAVELIN
--------------------
rm_type: {}
lagtobaseline: {}
laglimit: {}
fixed: {}
p_fix: {}
subtract_mean: {}
nwalker: {}
nburn: {}
nchain: {}
output_chains: {}
output_burn: {}
output_logp: {}
nbin: {}
metric: {}
together: {}
--------------------
        """.format( rm_type, lagtobaseline, laglimit_str, not (fixed is None), not (fixed is None),
                    subtract_mean, nwalkers, nburn, nchain, output_chains,
                    output_burn, output_logp, nbin, metric, together )

        print(txt_str)

    if together:
        res = utils.run_javelin(cont_fname, line_fnames, line_names,
                                rm_type=rm_type,
                                output_dir=output_dir + r'javelin/',
                                lagtobaseline=lagtobaseline, laglimit=laglimit,
                                fixed=fixed, p_fix=p_fix, subtract_mean=subtract_mean,
                                nwalkers=nwalkers, nburn=nburn, nchain=nchain, threads=threads,
                                output_chains=output_chains, output_burn=output_burn, output_logp=output_logp,
                                nbin=nbin, verbose=verbose, plot=plot)


        #Plot histograms
        fig, ax = plotting.plot_javelin_hist( res, fixed=fixed, nbin=nbin,
                                             time_unit=time_unit,
                                             remove_fixed=False,
                                             fname= output_dir + 'javelin/javelin_histogram.pdf' )

        if plot:
            plt.show()

        plt.cla()
        plt.clf()
        plt.close()


        if (fixed is None) | (fixed is np.ones( 2 + 3*len(line_fnames) )):

            #Corner plot
            fig, ax = plotting.javelin_corner(res,
                                            fname= output_dir + 'javelin/javelin_corner.pdf' )


            if plot:
                plt.show()

            plt.cla()
            plt.clf()
            plt.close()




        bestfit_model = utils.javelin_pred_lc( res['rmap_model'],
                                            res['tot_dat'].jlist[0], res['tot_dat'].jlist[1:],
                                            metric=metric)

        res['bestfit_model'] = bestfit_model


        #Plot model fits
        fig, ax = plotting.plot_javelin_bestfit(res, bestfit_model, time_unit=time_unit, lc_unit=lc_unit,
                                                fname= output_dir + 'javelin/javelin_bestfit.pdf' )

        if plot:
            plt.show()

        plt.cla()
        plt.clf()
        plt.close()




        #Write fits to light curves
        for i in range(len(line_names)):
            dat_fname = output_dir + r'javelin/' + line_names[i] + '_lc_fits.dat'
            dat = [ bestfit_model.jlist[i],
                    bestfit_model.mlist[i] + bestfit_model.blist[i],
                    bestfit_model.elist[i] ]
            write_data( dat, dat_fname )


        return res


    else:
        res_tot = []

        for i in range(len(line_fnames)):
            names_i = [line_names[0], line_names[i+1]]

            if isinstance(laglimit[i], str):
                input_laglimit = laglimit[i]
            else:
                input_laglimit = [laglimit[i]]

            res = utils.run_javelin(cont_fname, line_fnames[i], names_i,
                                    rm_type=rm_type,
                                    output_dir=output_dir + names_i[1] + r'/javelin/',
                                    lagtobaseline=lagtobaseline, laglimit=input_laglimit,
                                    fixed=fixed[i], p_fix=p_fix[i], subtract_mean=subtract_mean,
                                    nwalkers=nwalkers, nburn=nburn, nchain=nchain, threads=threads,
                                    output_chains=output_chains, output_burn=output_burn, output_logp=output_logp,
                                    verbose=verbose, plot=plot)


            #Plot histograms
            fig, ax = plotting.plot_javelin_hist( res, fixed=fixed[i], nbin=nbin,
                                                  time_unit=time_unit,
                                                  remove_fixed=False,
                                                  fname= output_dir + line_names[i+1] + r'/javelin/javelin_histogram.pdf' )

            if plot:
                plt.show()

            plt.cla()
            plt.clf()
            plt.close()



            #Corner plot
            if (fixed[i] is None) | (fixed[i] is np.ones( 2 + 3*len(line_fnames) )):
                fig, ax = plotting.javelin_corner(res,
                                                fname= output_dir + line_names[i+1] + '/javelin/javelin_corner.pdf' )

                if plot:
                    plt.show()

                plt.cla()
                plt.clf()
                plt.close()


            bestfit_model = utils.javelin_pred_lc( res['rmap_model'],
                                                res['tot_dat'].jlist[0], res['tot_dat'].jlist[1:],
                                                metric=metric)



            res['bestfit_model'] = bestfit_model
            res_tot.append(res)

            #Plot model fits
            fig, ax = plotting.plot_javelin_bestfit(res, bestfit_model, time_unit=time_unit,
                                                    lc_unit=[lc_unit[0], lc_unit[i+1]],
                                                    fname= output_dir + line_names[i+1] + '/javelin/javelin_bestfit.pdf' )

            if plot:
                plt.show()

            plt.cla()
            plt.clf()
            plt.close()



            #Write fits to light curves
            for j in range( bestfit_model.nlc ):

                if j == 0:
                    name = line_names[0]
                else:
                    name = line_names[i+1]

                dat_fname = output_dir + line_names[i+1] + r'/javelin/' + name + '_lc_fits.dat'
                dat = [ bestfit_model.jlist[j],
                        bestfit_model.mlist[j] + bestfit_model.blist[j],
                        bestfit_model.elist[j] ]
                write_data( dat, dat_fname )

        return res_tot